/**
 * @file main.c
 * @brief Este projeto demonstra o uso de temporizadores SysTick (nucleo), PIT e LPTMR
 * @author Wu Shin Ting
 * @date 23/10/2020
 */

#include "derivative.h" /* include peripheral declarations */

/**
 * @brief Inicializar os pinos de controle do led RGB
 */
void GPIO_initLedRGB() {
	// Modulo SIM: habilita o clock do modulo PORTE
	SIM_SCGC5 |= SIM_SCGC5_PORTE_MASK; 
	
	// Modulo PORT: configure o pino PTE21 para GPIO (vermelho)
	PORTE_PCR21 |= PORT_PCR_MUX (0b001); // mascarar os bits 8, 9 e 10
	
	// Modulo PORT: configure o pino PTE22 para GPIO (verde)
	PORTE_PCR22 |= PORT_PCR_MUX (0b001); // mascarar os bits 8, 9 e 10

	// Modulo PORT: configure o pino PTE23 para GPIO (azul)
	PORTE_PCR23 |= PORT_PCR_MUX (0b001); // mascarar os bits 8, 9 e 10

	// Modulo GPIO: configure o sentido do sinal nos pinos PTE21, PTE22 e PTE23
	GPIOE_PDDR |= (0b111<<21);    // Seta pinos 21, 22 e 23 do PORTE como saida 	
	
	// Modulo GPIO: inicializar PTE21, PTE22 e PTE23 em 0 (estado default no reset)
	GPIOE_PCOR = (0b111<<21);    // Resetar pinos 21, 22 e 23 do PORTE
}

/**
 * @brief Inicializar o temporizador SysTick com um valor de recarga. A base de tempo
 * eh o relogio do nucleo (processador), configurado em 20971529Hz
 * @param[in] periodo valor de recarga
 */
void SysTick_init (unsigned int periodo) {
	SYST_RVR = SysTick_RVR_RELOAD(periodo);   
	SYST_CVR = SysTick_CVR_CURRENT(0);          // reseta flag e recarga     
	SYST_CSR |= (SysTick_CSR_CLKSOURCE_MASK |	// fonte de CLK: nucleo
			SysTick_CSR_TICKINT_MASK |			// habilita interrupcao
			SysTick_CSR_ENABLE_MASK);         	// habilita SysTick 
}

void SysTick_Handler () {
	GPIOE_PTOR = (1<<21);	// alternar o estado de R do led RGB
}

/**
 * @brief Configurar o timer 0 do PIT para que gere pulsos periodicos para um dado valor de recarga.
 * A base de tempo eh o sinal de relogio do barramento (MCGOUTCLK/(OUTDIV1*OUTDIV4)), 
 * onde OUTDIV1=OUTDIV2=1.
 * @param[in] periodo valor de recarga
 */
void PIT_initTimer0(unsigned int periodo) {
	SIM_SCGC6 |= SIM_SCGC6_PIT_MASK;        // habilita CLK 
	
	SIM_CLKDIV1 &= ~SIM_CLKDIV1_OUTDIV4(0b111);  //prescaler = 1

	PIT_TCTRL0 &= ~(PIT_TCTRL_TEN_MASK); 		// desativar o timer 
	PIT_TCTRL0 &= ~PIT_TCTRL_CHN_MASK;   		// timers nao encadeados
	PIT_LDVAL0 = PIT_LDVAL_TSV(periodo);       	// carregar o valor de recarga

	PIT_TCTRL0 |= (PIT_TCTRL_TEN_MASK);  		// ativa timer 0
	
	PIT_MCR &= ~(PIT_MCR_FRZ_MASK |             // continua contagem no modo Debug)
				PIT_MCR_MDIS_MASK );     		// habilita modulo PIT
}

/**
 * @Habilitar o mecanismo de interrupcao para modulo PIT
 */
void PIT_enableNVICInterrupt(char priority) {
    NVIC_ISER |= 1 << (38-16);         	// NVIC_ISER[22]=1 (habilita IRQ22)
    NVIC_ICPR |= 1 << (38-16);         	// NVIC_ICPR[22]=1 (limpa as pendncias)
    NVIC_IPR5 |= NVIC_IP_PRI_22(priority << 6); // 22/4 = 5
    
	PIT_TFLG0 |= PIT_TFLG_TIF_MASK;     // w1c (limpa pendncias)
	PIT_TCTRL0 |= PIT_TCTRL_TIE_MASK;   // habilita interrupcao
}

/**
 * @brief tratar o evento de interrupcao gerado pelo modulo PIT
 */
void PIT_IRQHandler(void) {
	GPIOE_PTOR = (1<<22);				// alternar o estado de G do led RGB
	PIT_TFLG0 |= PIT_TFLG_TIF_MASK;     // w1c (limpa pendncias)
}

/**
 * @breif Inicailizar o modulo LPTMR0
 */
uint8_t LPTimer_init (unsigned short valor)
{
	SIM_SCGC5 |= SIM_SCGC5_LPTMR_MASK;  // habilita o sinal CLK para LPTMR0

	LPTMR0_CSR &= ~(LPTMR_CSR_TPP_MASK	// Polaridade: contador CNR incrementa na borda de subida (0) 
			| LPTMR_CSR_TFC_MASK  		// Contador CNR reseta quando CNR = CMR
			| LPTMR_CSR_TMS_MASK  		// Modo de contagem por tempo
	);

	 // Selecionar a base de tempo dentre MCGIRCLK(0b00)/LPO(0b01)/OSCERCLK(0b11)
	 // ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf#page=90
	LPTMR0_PSR |= LPTMR_PSR_PCS(0b01);	// fonte de base de tempo LPO (1kHz)

	LPTMR0_PSR &= ~LPTMR_PSR_PBYP_MASK;    
	LPTMR0_PSR |= LPTMR_PSR_PRESCALE(0b0000); 	// Prescaler = 2

	LPTMR0_CMR = LPTMR_CMR_COMPARE(valor);  	// referencia de contagem

	LPTMR0_CSR |= LPTMR_CSR_TEN_MASK;   		// habilitar LPTMR
	
	return 0;
}

void LPTimer_enableNVICInterrupt (char priority) {
    NVIC_ISER |= 1 << (44-16);         	// NVIC_ISER[28]=1 (habilita IRQ28)
    NVIC_ICPR |= 1 << (44-16);         	// NVIC_ICPR[28]=1 (limpa as pendncias)
    NVIC_IPR7 |= NVIC_IP_PRI_28(priority << 6); // 28/4 = 7			

	LPTMR0_CSR |= ( LPTMR_CSR_TCF_MASK   ///< Limpar pendencias (w1c)
                  | LPTMR_CSR_TIE_MASK   ///< Habilitar interrupcao
                  );
}

/**
 * @brief tratar o evento de interrupcao gerado pelo modulo LPTMR0
 */
void LPTimer_IRQHandler(void) {
	GPIOE_PTOR = (1<<23);				// alternar o estado de G do led RGB
		
	LPTMR0_CSR |= LPTMR_CSR_TCF_MASK;   // Limpar pendencias (w1c)
}

/**
 * @brief Inicializar e aguardar eventos num laco de espera
 */
int main(void)
{
	GPIO_initLedRGB(); 			// Inicializar os pinos de controle do led RGB
	
	SysTick_init(5242880);  	// Intervalo de contagem: 5242880/20971520 = 0.25s 	

	PIT_initTimer0(5242880); 	// Intervalo de contagem: 5242880/20971520 = 0.25s
	
	PIT_enableNVICInterrupt(3);		// Habilitar interrupcao para PIT
	
	LPTimer_init (125);			// 2*125/1000 = 0.25s
	
	LPTimer_enableNVICInterrupt (3); // Habilitar interrupcao para LPTMR
	
	for(;;) {	   
	}
	
	return 0;
}
